﻿using Styx.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HBProfileGenerator
{
    public class Profile
    {
        internal String Filepath;

        public List<ProfileTemplate> Templates { get; set; }
        public ProfileSettings Settings { get; set; }
        public bool LoopAllProfiles { get; set; }

        public Profile()
        {
            Settings = new ProfileSettings();
        }

        public Profile(String Filepath)
        {
            Logging.Write(LogLevel.Diagnostic, "Profile(Filepath) entered");

            this.Filepath = Filepath;

            //Read the file
            StreamReader sr = new StreamReader(Filepath);
            String Content = sr.ReadToEnd();
            sr.Close();

            Settings = new ProfileSettings(Content.Substring(0, Content.IndexOf("<QuestOrder>")));

            String[] ContentLines = Content.Split(new char[] { '\n' }).Where(w => w.Trim().StartsWith("<!-- #")).ToArray();

            Templates = new List<ProfileTemplate>();

            for (int i = 0; i < ContentLines.Count(); i++)
            {
                if(!ContentLines[i].ToLower().Contains("<!-- #id"))
                {
                    continue;
                }

                if(ProfileTemplate.All.Any(a => a.TemplateId == ContentLines[i].ToLower().Replace("<!-- #id", "").Replace("-->", "").Trim()))
                {
                    Templates.Add(ProfileTemplate.All.First(a => a.TemplateId == ContentLines[i].ToLower().Replace("<!-- #id", "").Replace("-->", "").Trim()));
                }
                else
                {
                    if(i > 1 && ContentLines[i-1].Contains("<!-- #Title"))
                    {
                        MessageBox.Show("Could not find a ProfileTemplate in the Profile.\nID:" + ContentLines[i].ToLower().Replace("<!-- #id", "").Replace("-->", "").Trim() + "\nEntrys: " + String.Join("\n->", ProfileTemplate.All));
                    }
                }
            }
            Logging.Write(LogLevel.Diagnostic, "Profile(Filepath) left");
        }

        public void Save()
        {
            Logging.Write(LogLevel.Diagnostic, "Save() entered");

            if(Filepath == null)
            {
                return;
            }

            StreamWriter sw = new StreamWriter(Filepath, false);

            sw.WriteLine("<HBProfile xsi:noNamespaceSchemaLocation=\"../../../Schemas/QuestProfileSchema.xsd\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
            sw.WriteLine(Settings.Content);
            sw.WriteLine("\t<QuestOrder>");
            if (LoopAllProfiles)
            {
                sw.WriteLine("\t\t<While Condition=\"True\">");
            }

            sw.WriteLine("\n<!-- " + new String('.', 10) + "Starting Profiles" + new String('.', 10) + " -->\n");
            Templates.ForEach(f => sw.WriteLine("\n" + f.Content + "\n"));
            sw.WriteLine("\n<!-- " + new String('.', 10) + "Ending Profiles" + new String('.', 10) + " -->\n");

            if (LoopAllProfiles)
            {
                sw.WriteLine("\t\t</While>");
            }
            sw.WriteLine("\t</QuestOrder>");
            sw.WriteLine("</HBProfile>");

            sw.Close();

            Logging.Write("saved to " + Filepath);

            Logging.Write(LogLevel.Diagnostic, "Save() left");
        }

        public override string ToString()
        {
            Logging.Write(LogLevel.Diagnostic, "ToString() called");
            return (Settings != null && Settings.Name != null) ? Settings.Name : "No Title";
        }
    }
}
