﻿using Styx.Common;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HBProfileGenerator
{
    public class ProfileSettings
    {
        public String Name { get; set; }
        public double MinDurability { get; set; }
        public int MinFreeBagSlots { get; set; }
        public int MinLevel { get; set; }
        public int MaxLevel { get; set; }
        public bool SellGrey { get; set; }
        public bool SellWhite { get; set; }
        public bool SellGreen { get; set; }
        public bool SellBlue { get; set; }
        public bool SellPurple { get; set; }
        public bool MailGrey { get; set; }
        public bool MailWhite { get; set; }
        public bool MailGreen { get; set; }
        public bool MailBlue { get; set; }
        public bool MailPurple { get; set; }
        public List<ProfileVendor> Vendors { get; set; }
        public List<String> Mailboxes { get; set; }

        public ProfileSettings()
        {
            Vendors = new List<ProfileVendor>();
            Mailboxes = new List<string>();
        }

        public ProfileSettings(String SettingsInput) : this()
        {
            Logging.Write(LogLevel.Diagnostic, "ProfileSettings(SettingsInput) entered");
            
            String[] Tags = SettingsInput.Split(new char[] { '<', '>' }).ToArray();
            for (int i = 0; i < Tags.Count(); i++)
            {
                switch (Tags[i].Contains(" ") ? Tags[i].Substring(0, Tags[i].IndexOf(" ")) : Tags[i])
                {
                    case "Name":
                        Name = Tags[i + 1];
                        break;
                    case "MinDurability":
                        MinDurability = Double.Parse(Tags[i + 1]);
                        break;
                    case "MinFreeBagSlots":
                        MinFreeBagSlots = Int32.Parse(Tags[i + 1]);
                        break;
                    case "MinLevel":
                        MinLevel = Int32.Parse(Tags[i + 1]);
                        break;
                    case "MaxLevel":
                        MaxLevel = Int32.Parse(Tags[i + 1]);
                        break;
                    case "SellGrey":
                        SellGrey = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "SellWhite":
                        SellWhite = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "SellGreen":
                        SellGreen = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "SellBlue":
                        SellBlue = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "SellPurple":
                        SellPurple = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "MailGrey":
                        MailGrey = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "MailWhite":
                        MailWhite = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "MailGreen":
                        MailGreen = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "MailBlue":
                        MailBlue = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "MailPurple":
                        MailPurple = (Tags[i + 1].ToLower() == "false");
                        break;
                    case "Vendor":
                        Vendors.Add(new ProfileVendor(Tags[i]));
                        break;
                    case "Mailbox":
                        Mailboxes.Add(Tags[i]);
                        break;
                    default:
                        break;
                }
            }

            Logging.Write(LogLevel.Diagnostic, "ProfileSettings(SettingsInput) left");
        }

        public String Content
        {
            get
            {
                Logging.Write(LogLevel.Diagnostic, "Content called");
                return
                    "\t<Name>" + Name + "</Name>\n\n" +
                    "\t<MinDurability>" + MinDurability + "</MinDurability>\n" +
                    "\t<MinFreeBagSlots>" + MinFreeBagSlots + "</MinFreeBagSlots>\n\n" +
                    "\t<MinLevel>" + MinLevel + "</MinLevel>\n" +
                    "\t<MaxLevel>" + MaxLevel + "</MaxLevel>\n\n" +
                    "\t<SellGrey>" + SellGrey + "</SellGrey>\n" +
                    "\t<SellWhite>" + SellWhite + "</SellWhite>\n" +
                    "\t<SellGreen>" + SellGreen + "</SellGreen>\n" +
                    "\t<SellBlue>" + SellBlue + "</SellBlue>\n" +
                    "\t<SellPurple>" + SellPurple + "</SellPurple>\n\n" +
                    "\t<MailGrey>" + MailGrey + "</MailGrey>\n" +
                    "\t<MailWhite>" + MailWhite + "</MailWhite>\n" +
                    "\t<MailGreen>" + MailGreen + "</MailGreen>\n" +
                    "\t<MailBlue>" + MailBlue + "</MailBlue>\n" +
                    "\t<MailPurple>" + MailPurple + "</MailPurple>\n\n" +
                    (Vendors.Count > 0 ? "\t<Vendors>" + String.Join("\n", Vendors.Select(s => "\t\t" + s.Content)) + "\t</Vendors>" : "") +
                    (Mailboxes.Count > 0 ? "\t<Vendors>" + String.Join("\n", Mailboxes.Select(s => "\t\t" + s)) + "\t</Mailboxes>" : "");
            }
        }
    }
}
