﻿using Styx.Common;
using Styx.CommonBot.Profiles;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HBProfileGenerator
{
    public partial class Settings_Main : Form
    {
        private Settings_Profile ProfileSettingsWindow;

        public static Settings_Main current;

        public Settings_Main()
        {
            InitializeComponent();
            current = this;
            CheckForIllegalCrossThreadCalls = false;
        }

        private void btnProfileEdit_Click(object sender, EventArgs e)
        {
            Logging.Write(LogLevel.Diagnostic, "btnProfileEdit_Click() entered");
            if(listProfiles.SelectedItem != null && listProfiles.SelectedItem is Profile)
            {
                if (ProfileSettingsWindow == null || !ProfileSettingsWindow.Visible)
                {
                    ProfileSettingsWindow = new Settings_Profile((Profile)listProfiles.SelectedItem);
                    ProfileSettingsWindow.Show();
                }
            }
            Logging.Write(LogLevel.Diagnostic, "btnProfileEdit_Click() left");
        }

        private void btnProfileNew_Click(object sender, EventArgs e)
        {
            Logging.Write(LogLevel.Diagnostic, "btnProfileNew_Click() entered");

            if (ProfileSettingsWindow == null || !ProfileSettingsWindow.Visible)
            {
                ProfileSettingsWindow = new Settings_Profile();
                ProfileSettingsWindow.Show();
            }

            Logging.Write(LogLevel.Diagnostic, "btnProfileNew_Click() left");
        }

        private void btnProfileLoad_Click(object sender, EventArgs e)
        {
            if (listProfiles.SelectedItem != null && listProfiles.SelectedItem is Profile)
            {
                Logging.Write(LogLevel.Normal, "--- Loading Profile ---");
                ProfileManager.LoadNew(((Profile)listProfiles.SelectedItem).Filepath);
                Logging.Write(LogLevel.Normal, "--- You can start the Bot now ---");
            }
        }

        private void Settings_Main_Load(object sender, EventArgs e)
        {
            Logging.Write(LogLevel.Diagnostic, "Settings_Main_Load() entered");

            Panel pCurtain = new Panel();
            pCurtain.BackColor = Color.DarkGray;
            pCurtain.Dock = DockStyle.Fill;
            this.Controls.Add(pCurtain);

            Thread t = new Thread(new ThreadStart(() =>
                {

                    if (!Updater.IsUpToDate && MessageBox.Show("There is a new Version available. You want me to update now?", "", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
                    {
                        Updater.UpdateClient();
                        return;
                    }


                    NeokilSettings.Current.Load();

                    #region Load ProfileTemplates
                    try
                    {
                        Directory.GetFiles(Path.Combine(Utilities.AssemblyDirectory, "Plugins\\HBProfileGenerator\\Templates")).Where(w => w.Contains("tmpl")).ForEach(f =>
                        {
                            try
                            {
                                if (!ProfileTemplate.All.Any(a => a.Filename == f))
                                {
                                    new ProfileTemplate(f);
                                }
                            }
                            catch
                            { }
                        });
                    }
                    catch (Exception ex)
                    {
                        Logging.WriteException(LogLevel.Diagnostic, ex);
                    }
                    #endregion

                    #region Load Profiles

                    RefreshProfileList();

                    #endregion

                    #region Load "Fishing"

                    checkFishingPluginDeleteDublicates.Checked = NeokilSettings.Current.FishingPluginDeleteDublicates;
                    checkFishingPluginAutoGutAndClean.Checked = NeokilSettings.Current.FishingPluginAutoGutAndClean;

                    #endregion

                    #region Load "LootFilter"

                    if (NeokilSettings.Current.LootFilterPluginRemoveItemEntrys != null)
                    {
                        listLootFilterPluginRemoveItems.Items.AddRange(NeokilSettings.Current.LootFilterPluginRemoveItemEntrys.Select(s => ItemInfo.FromId(s)).ToArray());
                    }
                    if (NeokilSettings.Current.LootFilterPluginUseItemEntrys != null)
                    {
                        listLootFilterPluginUseItems.Items.AddRange(NeokilSettings.Current.LootFilterPluginUseItemEntrys.Select(s => ItemInfo.FromId(s)).ToArray());
                    }

                    listLootFilterPluginMyBags.Items.AddRange(Styx.StyxWoW.Me.BagItems.Select(s => s.ItemInfo).ToArray());

                    #endregion

                    this.Controls.Remove(pCurtain);
                }));
            t.Start();
            
            Logging.Write(LogLevel.Diagnostic, "Settings_Main_Load() left");
        }

        private void Settings_Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            Logging.Write(LogLevel.Diagnostic, "Settings_Main_FormClosing() entered");

            #region Save Settings "Fishing"

            NeokilSettings.Current.FishingPluginDeleteDublicates = checkFishingPluginDeleteDublicates.Checked;
            NeokilSettings.Current.FishingPluginAutoGutAndClean = checkFishingPluginAutoGutAndClean.Checked;

            #endregion

            #region Save Settings "LootFilter"

            NeokilSettings.Current.LootFilterPluginRemoveItemEntrys = listLootFilterPluginRemoveItems.Items.Cast<ItemInfo>().Select(s => s.Id).ToArray();
            NeokilSettings.Current.LootFilterPluginUseItemEntrys = listLootFilterPluginUseItems.Items.Cast<ItemInfo>().Select(s => s.Id).ToArray();

            #endregion

            NeokilSettings.Current.Save();

            Logging.Write(LogLevel.Diagnostic, "Settings_Main_FormClosing() left");
        }

        private void toolStripStatusLabel1_Click(object sender, EventArgs e)
        {
            Process.Start("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=LSVESZA6MBAMW");
        }

        private void btnLootFilterPluginRemoveItemsRemove_Click(object sender, EventArgs e)
        {
            if (listLootFilterPluginRemoveItems.SelectedItem != null && listLootFilterPluginRemoveItems.SelectedItem is ItemInfo)
            {
                listLootFilterPluginRemoveItems.Items.Remove(listLootFilterPluginRemoveItems.SelectedItem);
            }
        }

        private void btnLootFilterPluginRemoveItemsAdd_Click(object sender, EventArgs e)
        {
            if (listLootFilterPluginMyBags.SelectedItem != null &&
                listLootFilterPluginMyBags.SelectedItem is ItemInfo &&
                !listLootFilterPluginRemoveItems.Items.Contains(listLootFilterPluginMyBags.SelectedItem))
            {
                listLootFilterPluginRemoveItems.Items.Add(listLootFilterPluginMyBags.SelectedItem);
            }
        }

        private void btnLootFilterPluginUseItemsRemove_Click(object sender, EventArgs e)
        {
            if (listLootFilterPluginUseItems.SelectedItem != null && listLootFilterPluginUseItems.SelectedItem is ItemInfo)
            {
                listLootFilterPluginUseItems.Items.Remove(listLootFilterPluginUseItems.SelectedItem);
            }
        }

        private void btnLootFilterPluginUseItemsAdd_Click(object sender, EventArgs e)
        {
            if (listLootFilterPluginMyBags.SelectedItem != null && 
                listLootFilterPluginMyBags.SelectedItem is ItemInfo &&
                !listLootFilterPluginUseItems.Items.Contains(listLootFilterPluginMyBags.SelectedItem))
            {
                listLootFilterPluginUseItems.Items.Add(listLootFilterPluginMyBags.SelectedItem);
            }
        }

        private void listLootFilterPluginRemoveItems_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            btnLootFilterPluginRemoveItemsRemove_Click(sender, e);
        }

        private void listLootFilterPluginUseItems_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            btnLootFilterPluginUseItemsRemove_Click(sender, e);
        }

        public void RefreshProfileList()
        {
            try
            {
                listProfiles.Items.Clear();
                Directory.GetFiles(Path.Combine(Utilities.AssemblyDirectory, "Plugins\\HBProfileGenerator\\Profiles"), "*.xml").ForEach(f =>
                {
                    Logging.Write("We found a profile...");
                    try
                    {
                        listProfiles.Items.Add(new Profile(f));
                    }
                    catch (Exception ex)
                    {
                        Logging.Write(ex.Message);
                    }
                });
            }
            catch (Exception ex)
            {
                Logging.WriteException(LogLevel.Diagnostic, ex);
            }
        }

        private void listProfiles_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            btnProfileEdit_Click(sender, e);
        }

        private void btnProfileRemove_Click(object sender, EventArgs e)
        {
            Logging.Write(LogLevel.Diagnostic, "btnProfileRemove_Click() entered");
            if (listProfiles.SelectedItem != null && listProfiles.SelectedItem is Profile)
            {
                File.Delete((listProfiles.SelectedItem as Profile).Filepath);
                listProfiles.Items.Remove(listProfiles.SelectedItem);
            }
            Logging.Write(LogLevel.Diagnostic, "btnProfileRemove_Click() left");
        }
    }
}
