﻿using Styx.Common;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace HBProfileGenerator
{
    public partial class Settings_Profile : Form
    {
        private Profile Profile;

        public Settings_Profile()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            Logging.Write(LogLevel.Diagnostic, "OnLoad() entered");

            if(Profile == null)
            {
                try
                {
                    this.Profile = new Profile();
                }
                catch (Exception ex)
                {
                    Logging.WriteException(LogLevel.Diagnostic, ex);
                }
            }
            else
            {
                try
                {
                    tbProfilname.Text = Profile.Settings.Name;
                    trackDurability.Value = (int)(Profile.Settings.MinDurability * 100);
                    nudFreeBagSlots.Value = Profile.Settings.MinFreeBagSlots;

                    nudMinLevel.Value = Profile.Settings.MinLevel;
                    nudMaxLevel.Value = Profile.Settings.MaxLevel;

                    checkSellingGrey.Checked = Profile.Settings.SellGrey;
                    checkSellingWhite.Checked = Profile.Settings.SellWhite;
                    checkSellingGreen.Checked = Profile.Settings.SellGreen;
                    checkSellingBlue.Checked = Profile.Settings.SellBlue;
                    checkSellingPurple.Checked = Profile.Settings.SellPurple;

                    checkMailingGrey.Checked = Profile.Settings.MailGrey;
                    checkMailingWhite.Checked = Profile.Settings.MailWhite;
                    checkMailingGreen.Checked = Profile.Settings.MailGreen;
                    checkMailingBlue.Checked = Profile.Settings.MailBlue;
                    checkMailingPurple.Checked = Profile.Settings.MailPurple;

                    listVendors.Items.AddRange(Profile.Settings.Vendors.ToArray());
                    listMailboxes.Items.AddRange(Profile.Settings.Mailboxes.ToArray());
                    listProfilTemplates.Items.AddRange(Profile.Templates.ToArray());

                    checkLoopProfiles.Checked = Profile.LoopAllProfiles;
                }
                catch (Exception ex)
                {
                    Logging.WriteException(LogLevel.Diagnostic, ex);
                }
            }

            listProfileAvailable.DataSource = ProfileTemplate.All;

            Logging.Write(LogLevel.Diagnostic, "OnLoad() left");
        }

        public Settings_Profile(Profile Profile) : this()
        {
            this.Profile = Profile;
        }

        #region UI-Events

        private void trackDurability_Scroll(object sender, EventArgs e)
        {
            lblDurability.Text = trackDurability.Value + "%";
        }
        
        private void tbProfileAvailableFilter_TextChanged(object sender, EventArgs e)
        {
            listProfileAvailable.DataSource = ProfileTemplate.All.Where(w => w.Title.ToLower().Contains(tbProfileAvailableFilter.Text.ToLower()));
        }

        private void btnVendorsAdd_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(tbVendorNewName.Text) &&
                !String.IsNullOrEmpty(tbVendorNewId.Text) &&
                !String.IsNullOrEmpty(tbVendorNewX.Text) &&
                !String.IsNullOrEmpty(tbVendorNewY.Text) &&
                !String.IsNullOrEmpty(tbVendorNewZ.Text))
            {
                listVendors.Items.Add(new ProfileVendor("<Vendor Name=\"" + tbVendorNewName.Text + "\" Entry=\"" + tbVendorNewId.Text + "\" Type=\"" + cmbVendorNewType.Text + "\" X=\"" + tbVendorNewX.Text + "\" Y=\"" + tbVendorNewY.Text + "\" Z=\"" + tbVendorNewZ.Text + "\" />"));
            }
            else
            {
                MessageBox.Show("Some values seem to be missing but are required to add a vendor.");
            }
        }

        private void btnVendorsRemove_Click(object sender, EventArgs e)
        {
            if (listVendors.SelectedItem != null && listVendors.SelectedItem is ProfileVendor)
            {
                listVendors.Items.Remove(listVendors.SelectedItem);
            }
        }
        
        private void btnMailboxAdd_Click(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(tbMailboxesNewX.Text) &&
                String.IsNullOrEmpty(tbMailboxesNewY.Text) &&
                String.IsNullOrEmpty(tbMailboxesNewZ.Text))
            {
                listMailboxes.Items.Add("<Mailbox X=\"" + tbMailboxesNewX.Text + "\" Y=\"" + tbMailboxesNewX.Text + "\" Z=\"" + tbMailboxesNewX.Text + "\" />");
            }
        }

        private void btnMailboxRemove_Click(object sender, EventArgs e)
        {
            if (listMailboxes.SelectedItem != null)
            {
                listMailboxes.Items.Remove(listMailboxes.SelectedItem);
            }
        }

        private void btnProfileMoveUp_Click(object sender, EventArgs e)
        {
            var move = listProfilTemplates.SelectedItem;
            var index = listProfilTemplates.SelectedIndex;
            if (index > 0)
            {
                listProfilTemplates.Items.Remove(move);
                listProfilTemplates.Items.Insert(index - 1, move);
                listProfilTemplates.SelectedIndex = index - 1;
            }
        }

        private void btnProfileMoveDown_Click(object sender, EventArgs e)
        {
            var move = listProfilTemplates.SelectedItem;
            var index = listProfilTemplates.SelectedIndex;
            if (index < listProfilTemplates.Items.Count - 1)
            {
                listProfilTemplates.Items.Remove(move);
                listProfilTemplates.Items.Insert(index + 1, move);
                listProfilTemplates.SelectedIndex = index + 1;
            }
        }

        private void btnProfileAdd_Click(object sender, EventArgs e)
        {
            if (listProfileAvailable.SelectedItem != null && listProfileAvailable.SelectedItem is ProfileTemplate)
            {
                if (listProfilTemplates.SelectedItem != null)
                {
                    listProfilTemplates.Items.Insert(listProfilTemplates.SelectedIndex + 1, listProfileAvailable.SelectedItem);
                }
                else
                {
                    listProfilTemplates.Items.Add(listProfileAvailable.SelectedItem);
                }
            }
        }

        private void btnProfileRemove_Click(object sender, EventArgs e)
        {
            if (listProfilTemplates.SelectedItem != null)
            {
                listProfilTemplates.Items.Remove(listProfilTemplates.SelectedItem);
            }
        }

        #endregion

        private void button1_Click(object sender, EventArgs e)
        {
            Logging.Write(LogLevel.Diagnostic, "button1_Click() entered");
            try
            {
                Profile.Settings.Name = tbProfilname.Text;
                Profile.Settings.MinDurability = trackDurability.Value / 100;
                Profile.Settings.MinFreeBagSlots = (int)nudFreeBagSlots.Value;

                Profile.Settings.MinLevel = (int)nudMinLevel.Value;
                Profile.Settings.MaxLevel = (int)nudMaxLevel.Value;

                Profile.Settings.SellGrey = checkSellingGrey.Checked;
                Profile.Settings.SellWhite = checkSellingWhite.Checked;
                Profile.Settings.SellGreen = checkSellingGreen.Checked;
                Profile.Settings.SellBlue = checkSellingBlue.Checked;
                Profile.Settings.SellPurple = checkSellingPurple.Checked;

                Profile.Settings.MailGrey = checkMailingGrey.Checked;
                Profile.Settings.MailWhite = checkMailingWhite.Checked;
                Profile.Settings.MailGreen = checkMailingGreen.Checked;
                Profile.Settings.MailBlue = checkMailingBlue.Checked;
                Profile.Settings.MailPurple = checkMailingPurple.Checked;

                Profile.Settings.Vendors = listVendors.Items.Cast<ProfileVendor>().ToList();
                Profile.Settings.Mailboxes = listMailboxes.Items.Cast<String>().ToList();
                Profile.Templates = listProfilTemplates.Items.Cast<ProfileTemplate>().ToList();

                Profile.LoopAllProfiles = checkLoopProfiles.Checked;

                if(Profile.Filepath == null)
                {
                    Profile.Filepath = Path.Combine(
                        Utilities.AssemblyDirectory, 
                        "Plugins\\HBProfileGenerator\\Profiles\\",  
                        (String.Join("", Profile.Settings.Name.Split(Path.GetInvalidFileNameChars(), StringSplitOptions.RemoveEmptyEntries)) + ".xml"));
                    
                    /*
                    var fileinput = new Settings_FilenameInput();
                    if(fileinput.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        Profile.Filepath = Path.Combine(Utilities.AssemblyDirectory, "Plugins\\HBProfileGenerator\\Profiles\\",  (fileinput.Value.EndsWith(".xml") ? fileinput.Value : fileinput.Value + ".xml"));
                    }*/
                }

                if(Profile.Filepath != null)
                {
                    Profile.Save();
                    Logging.Write("The Profile has been saved.");
                    this.Close();
                    Settings_Main.current.RefreshProfileList();
                }
            }
            catch (Exception ex)
            {
                Logging.WriteException(LogLevel.Diagnostic, ex);
            }
            Logging.Write(LogLevel.Diagnostic, "button1_Click() left");
        }

        private void listProfileAvailable_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            btnProfileAdd_Click(sender, e);
        }

        private void listProfilTemplates_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            btnProfileRemove_Click(sender, e);
        }
    }
}
